"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCurrentTheme = void 0;
var lib_1 = require("../../lib");
/**
 * Determines if the user has permissions, based on the permission kind value
 */
exports.getCurrentTheme = function () {
    // Return a promise
    return new Promise(function (resolve, reject) {
        // Wait for the modern theme information to be loaded
        waitForModernTheme().then(
        // Success
        function () {
            // Resolve the request with the theme information
            resolve(window["__themeState__"].theme);
        }, 
        // Error
        function () {
            // Get the current theme info
            lib_1.Web().Lists("Composed Looks").Items().query({
                Filter: "DisplayOrder eq 0 or Title eq 'Office'",
                OrderBy: ["DisplayOrder"],
                Select: ["DisplayOrder", "MasterPageUrl", "ThemeUrl", "Title"]
            }).execute(function (items) {
                var currentItem = items.results[0];
                var defaultItem = items.results[1];
                // See if the current theme info exists
                if (currentItem && currentItem["ThemeUrl"]) {
                    // Get the theme information
                    getThemeInfo(currentItem["ThemeUrl"].Url).then(resolve, reject);
                }
                else if (defaultItem && defaultItem["ThemeUrl"]) {
                    // Get the theme information
                    getThemeInfo(defaultItem["ThemeUrl"].Url).then(resolve, reject);
                }
                else {
                    // Unable to determine the theme
                    reject();
                }
            }, reject);
        });
    });
};
// Gets the theme information for a color palette
var getThemeInfo = function (url) {
    if (url === void 0) { url = ""; }
    // Return a promise
    return new Promise(function (resolve, reject) {
        var themeInfo = {};
        // Ensure the url exists
        if (url.length == 0) {
            resolve(themeInfo);
            return;
        }
        // Get the file item
        lib_1.Site().RootWeb().getFileByUrl(url).execute(function (file) {
            // Read the contents
            file.content().execute(function (contents) {
                // Convert the xml
                var parser = new DOMParser();
                var xmlDoc = parser.parseFromString(String.fromCharCode.apply(null, new Uint8Array(contents)), "text/xml");
                // Get the colors and parse them
                var colors = xmlDoc.getElementsByTagName("s:color");
                for (var i = 0; i < colors.length; i++) {
                    var color = colors[i];
                    var key = color.getAttribute("name");
                    var value = color.getAttribute("value");
                    // See if the length is > 6 characters
                    if (value.length > 6) {
                        // Convert the value
                        value = value.slice(2, 8) + value[0] + value[1];
                    }
                    // Add the color information
                    themeInfo[key] = "#" + value;
                }
                // Resolve the request
                resolve(themeInfo);
            }, reject);
        }, reject);
    });
};
// Waits for the modern theme information to be loaded
var waitForModernTheme = function () {
    // Return a promise
    return new Promise(function (resolve, reject) {
        var counter = 0;
        var maxAttempts = 100;
        // See if the modern theme exists
        if (window["__themeState__"] != null && window["__themeState__"].theme != null) {
            // Theme is loaded
            resolve();
            return;
        }
        // Wait for the theme to be loaded
        var loopId = setInterval(function () {
            // See if the modern theme exists
            if (window["__themeState__"] != null && window["__themeState__"].theme != null) {
                // Stop the loop
                clearInterval(loopId);
                // Resolve the request
                resolve();
            }
            // Else, see if we have hit the max attempts
            else if (++counter >= maxAttempts) {
                // Stop the loop
                clearInterval(loopId);
                // Reject the request
                reject();
            }
        }, 50);
    });
};
