"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.List = void 0;
var sptypes_1 = require("../sptypes");
var utils_1 = require("../utils");
var graph_1 = require("./graph");
var web_1 = require("./web");
/**
 * List
 */
exports.List = (function (listName, targetInfo) {
    var list = new utils_1.Base(targetInfo);
    // Default the properties
    list.targetInfo.defaultToWebFl = true;
    list.targetInfo.endpoint = "web/lists/getByTitle('" + listName.replace(/\'/g, "''") + "')";
    // Add the methods
    utils_1.Request.addMethods(list, { __metadata: { type: "SP.List" } });
    // Return the list
    return list;
});
// Static method to get the list by the entity name.
exports.List.getByEntityName = (function (props) {
    // Query for the list
    var query = web_1.Web(props.url, props.targetInfo)
        // Get the lists
        .Lists()
        // Set the query
        .query({
        Filter: "EntityTypeName eq '" + props.name + "'",
        Top: 1
    });
    // See if the callback exists
    if (props.callback) {
        // Execute the request asynchronously
        query.execute(function (lists) {
            // Execute the callback method
            props.callback(lists.results ? lists.results[0] : null);
        });
    }
    else {
        // Execute the request synchronously and return it
        var list = query.executeAndWait();
        return list.results ? list.results[0] : list;
    }
});
// Static method to get the list data from the SP.List.getListDataAsStream endpoint
exports.List.getDataAsStream = (function (listFullUrl, parameters) {
    if (parameters === void 0) { parameters = {}; }
    var params = "?listFullUrl='" + listFullUrl + "'";
    // Parse the parameters
    for (var key in parameters) {
        // Append the parameter
        params += "&" + key + "=" + parameters[key];
    }
    // Return the base object
    return new utils_1.Base({
        endpoint: "SP.List.getListDataAsStream" + params
    });
});
// Static method for executing a flow against a list item
exports.List.runFlow = function (props) {
    // Return a promise
    return new Promise(function (resolve) {
        // Gets the graph token
        var getGraphToken = function () {
            // Return a promise
            return new Promise(function (resolveAuth) {
                // Get the graph token
                graph_1.Graph.getAccessToken(sptypes_1.SPTypes.CloudEnvironment.Flow).execute(function (auth) {
                    // Resolve the request
                    resolveAuth(auth.access_token);
                }, 
                // Error
                function (ex) {
                    // Resolve the request
                    resolve({
                        executed: false,
                        errorDetails: ex.response,
                        errorMessage: "Auth Error: Unable to get the flow token."
                    });
                });
            });
        };
        // Gets the flow token
        var getFlowToken = function (flowInfo) {
            // Return a promise
            return new Promise(function (resolveAuth) {
                // See if the flow token is provided
                if (props.token) {
                    // Resolve the request
                    resolveAuth(props.token);
                }
                else {
                    // Get the graph token
                    getGraphToken().then(function (token) {
                        // See if the cloud environment was provided
                        if (props.cloudEnv) {
                            // Set the url
                            var authUrl = "" + props.cloudEnv + flowInfo.properties.environment.id + "/users/me/onBehalfOfTokenBundle?api-version=2016-11-01";
                            // Execute the request
                            new utils_1.Base({
                                endpoint: authUrl,
                                method: "POST",
                                requestType: utils_1.RequestType.GraphPost,
                                requestHeader: {
                                    "authorization": "Bearer " + token
                                }
                            }).execute(function (tokenInfo) {
                                // Resolve the request
                                resolveAuth(tokenInfo.audienceToToken["https://" + flowInfo.properties.connectionReferences.shared_sharepointonline.swagger.host] || token);
                            }, 
                            // Error
                            function (ex) {
                                // Resolve the request
                                resolve({
                                    executed: false,
                                    errorDetails: ex.response,
                                    errorMessage: "Auth Error: Unable to get the flow token for cloud: " + props.cloudEnv
                                });
                            });
                        }
                        else {
                            // Resolve the request
                            resolveAuth(token);
                        }
                    });
                }
            });
        };
        // Get the flow information
        web_1.Web(props.webUrl).Lists(props.list).syncFlowInstance(props.id).execute(
        // Success
        function (flow) {
            // Get the flow information
            var flowInfo = JSON.parse(flow.SynchronizationData);
            // Get the flow token
            getFlowToken(flowInfo).then(function (token) {
                // Trigger the flow
                new utils_1.Base({
                    accessToken: token,
                    requestType: utils_1.RequestType.GraphPost,
                    endpoint: flowInfo.properties.flowTriggerUri,
                    data: {
                        rows: [{
                                entity: props.data
                            }]
                    }
                }).execute(
                // Success
                function () {
                    // Resolve the request
                    resolve({
                        executed: true,
                        flowToken: token
                    });
                }, 
                // Error
                function (ex) {
                    // Resolve the request
                    resolve({
                        executed: false,
                        errorDetails: ex.response,
                        errorMessage: "Error triggering the flow."
                    });
                });
            });
        }, 
        // Error
        function (ex) {
            // Resolve the request
            resolve({
                executed: false,
                errorDetails: ex.response,
                errorMessage: "Error getting the flow information."
            });
        });
    });
};
