"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Search = void 0;
var utils_1 = require("../utils");
/**
 * Search
 */
exports.Search = (function (url, targetInfo) {
    var base = new utils_1.Base(targetInfo);
    var search = base;
    // Default the properties
    base.targetInfo.defaultToWebFl = true;
    base.targetInfo.endpoint = "search";
    // See if the web url exists
    if (url) {
        // Set the settings
        base.targetInfo.url = url;
    }
    // Add the methods
    utils_1.Request.addMethods(search, { __metadata: { type: "Microsoft.Office.Server.Search.REST.SearchService" } });
    /** The search query method */
    search.searchquery = function (settings) {
        // Execute the request
        return search.executeMethod("query", {
            argNames: ["query"],
            name: "query?[[query]]",
            requestType: utils_1.RequestType.GetReplace
        }, exports.Search.getQuery(settings));
    };
    // Return the search
    return search;
});
// Static method to compute the query
exports.Search.getQuery = function (parameters) {
    var query = "";
    // Parse the parameters
    for (var key in parameters) {
        // Append the parameter to the query
        query += (query == "" ? "" : "&") + key + "='" + parameters[key] + "'";
    }
    // Return the query
    return [query];
};
// Static post query method
exports.Search.postQuery = function (props) {
    // Return a promise
    return new Promise(function (resolve, reject) {
        var queryProps = props.query;
        // Compute the row count
        var rowCount = 500;
        if (typeof (queryProps.RowLimit) === "number") {
            // Set the custom limit
            rowCount = queryProps.RowLimit;
        }
        else {
            // Default to the max size
            queryProps.RowLimit = rowCount;
        }
        // Query the first batch
        exports.Search(props.url, props.targetInfo).postquery(queryProps).execute(
        // Success
        function (request) {
            // Updates the table
            var updateRequest = function (searchResult) {
                // Ensure the results exist
                if (searchResult) {
                    // Call the event
                    props.onQueryCompleted ? props.onQueryCompleted(searchResult) : null;
                    // Parse rows
                    for (var i = 0; i < searchResult.PrimaryQueryResult.RelevantResults.Table.Rows.results.length; i++) {
                        var row = searchResult.PrimaryQueryResult.RelevantResults.Table.Rows.results[i];
                        // Append the row
                        request.postquery.PrimaryQueryResult.RelevantResults.Table.Rows.results.push(row);
                    }
                }
            };
            // Call the event
            props.onQueryCompleted ? props.onQueryCompleted(request.postquery) : null;
            // See if more results exist
            var results = request.postquery.PrimaryQueryResult.RelevantResults;
            if (results.TotalRows > results.RowCount) {
                var search = exports.Search(props.url, props.targetInfo);
                var useBatch = typeof (props.useBatch) === "boolean" ? props.useBatch : true;
                // Compute the total # of requests that we need to make
                var totalPages = Math.ceil(results.TotalRows / rowCount);
                // Loop for the total # of requests
                for (var i = 1; i < totalPages; i++) {
                    // Set the start row
                    queryProps.StartRow = i * rowCount;
                    // See if we are making a batch request
                    if (useBatch) {
                        // Create a batch request
                        search.postquery(queryProps).batch(
                        // Success
                        function (batchRequest) {
                            // Update the request
                            updateRequest(batchRequest.postquery);
                        }, 
                        // Limit to 100 per request
                        i % 100 == 0);
                    }
                    else {
                        // Create the request
                        search.postquery(queryProps).execute(
                        // Success
                        function (batchRequest) {
                            // Update the request
                            updateRequest(batchRequest.postquery);
                        }, 
                        // Wait for the previous request to complete
                        true);
                    }
                }
                // See if we are making a batch request
                if (useBatch) {
                    // Execute the batch requests
                    search.execute(function () {
                        // Resolve the request
                        resolve(request.postquery);
                    }, reject);
                }
                else {
                    // Wait for the requests to complete
                    search.done(function () {
                        // Resolve the request
                        resolve(request.postquery);
                    });
                }
            }
            else {
                // Resolve the request
                resolve(request.postquery);
            }
        }, 
        // Error
        reject);
    });
};
